@@ -20,6 +20,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // Qt
 #include <QtTest/QtTest>
 // KWin
+#include "../../src/client/blur.h"
+#include "../../src/client/contrast.h"
 #include "../../src/client/compositor.h"
 #include "../../src/client/connection_thread.h"
 #include "../../src/client/dpms.h"
@@ -28,16 +30,19 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/output.h"
 #include "../../src/client/pointerconstraints.h"
 #include "../../src/client/pointergestures.h"
+#include "../../src/client/idleinhibit.h"
 #include "../../src/client/seat.h"
 #include "../../src/client/relativepointer.h"
 #include "../../src/client/server_decoration.h"
 #include "../../src/client/shell.h"
+#include "../../src/client/surface.h"
 #include "../../src/client/subcompositor.h"
 #include "../../src/client/xdgshell.h"
 #include "../../src/server/compositor_interface.h"
 #include "../../src/server/datadevicemanager_interface.h"
 #include "../../src/server/display.h"
 #include "../../src/server/dpms_interface.h"
+#include "../../src/server/idleinhibit_interface.h"
 #include "../../src/server/output_interface.h"
 #include "../../src/server/seat_interface.h"
 #include "../../src/server/shell_interface.h"
@@ -56,6 +61,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // Wayland
 #include <wayland-client-protocol.h>
 #include <wayland-dpms-client-protocol.h>
+#include <wayland-idle-inhibit-unstable-v1-client-protocol.h>
 #include <wayland-server-decoration-client-protocol.h>
 #include <wayland-text-input-v0-client-protocol.h>
 #include <wayland-text-input-v2-client-protocol.h>
@@ -92,9 +98,11 @@ private Q_SLOTS:
     void testBindRelativePointerManagerUnstableV1();
     void testBindPointerGesturesUnstableV1();
     void testBindPointerConstraintsUnstableV1();
+    void testBindIdleIhibitManagerUnstableV1();
     void testGlobalSync();
     void testGlobalSyncThreaded();
     void testRemoval();
+    void testOutOfSyncRemoval();
     void testDestroy();
     void testAnnounceMultiple();
     void testAnnounceMultipleOutputDevices();
@@ -116,6 +124,10 @@ private:
     KWayland::Server::RelativePointerManagerInterface *m_relativePointerV1;
     KWayland::Server::PointerGesturesInterface *m_pointerGesturesV1;
     KWayland::Server::PointerConstraintsInterface *m_pointerConstraintsV1;
+    KWayland::Server::BlurManagerInterface *m_blur;
+    KWayland::Server::ContrastManagerInterface *m_contrast;
+    KWayland::Server::IdleInhibitManagerInterface *m_idleInhibit;
+
 };
 
 static const QString s_socketName = QStringLiteral("kwin-test-wayland-registry-0");
@@ -138,6 +150,9 @@ TestWaylandRegistry::TestWaylandRegistry(QObject *parent)
     , m_relativePointerV1(nullptr)
     , m_pointerGesturesV1(nullptr)
     , m_pointerConstraintsV1(nullptr)
+    , m_blur(nullptr)
+    , m_contrast(nullptr)
+    , m_idleInhibit(nullptr)
 {
 }
 
@@ -164,8 +179,10 @@ void TestWaylandRegistry::init()
     m_outputDevice = m_display->createOutputDevice();
     m_outputDevice->create();
     QVERIFY(m_outputManagement->isValid());
-    m_display->createBlurManager(this)->create();
-    m_display->createContrastManager(this)->create();
+    m_blur = m_display->createBlurManager(this);
+    m_blur->create();
+    m_contrast = m_display->createContrastManager(this);
+    m_contrast->create();
     m_display->createSlideManager(this)->create();
     m_display->createDpmsManager()->create();
     m_serverSideDecorationManager = m_display->createServerSideDecorationManager();
@@ -188,6 +205,9 @@ void TestWaylandRegistry::init()
     m_pointerConstraintsV1 = m_display->createPointerConstraints(KWayland::Server::PointerConstraintsInterfaceVersion::UnstableV1);
     m_pointerConstraintsV1->create();
     QCOMPARE(m_pointerConstraintsV1->interfaceVersion(), KWayland::Server::PointerConstraintsInterfaceVersion::UnstableV1);
+    m_idleInhibit = m_display->createIdleInhibitManager(KWayland::Server::IdleInhibitManagerInterfaceVersion::UnstableV1);
+    m_idleInhibit->create();
+    QCOMPARE(m_idleInhibit->interfaceVersion(), KWayland::Server::IdleInhibitManagerInterfaceVersion::UnstableV1);
 }
 
 void TestWaylandRegistry::cleanup()
@@ -345,6 +365,11 @@ void TestWaylandRegistry::testBindPointerConstraintsUnstableV1()
     TEST_BIND(KWayland::Client::Registry::Interface::PointerConstraintsUnstableV1, SIGNAL(pointerConstraintsUnstableV1Announced(quint32,quint32)), bindPointerConstraintsUnstableV1, zwp_pointer_constraints_v1_destroy)
 }
 
+void TestWaylandRegistry::testBindIdleIhibitManagerUnstableV1()
+{
+    TEST_BIND(KWayland::Client::Registry::Interface::IdleInhibitManagerUnstableV1, SIGNAL(idleInhibitManagerUnstableV1Announced(quint32,quint32)), bindIdleInhibitManagerUnstableV1, zwp_idle_inhibit_manager_v1_destroy)
+}
+
 #undef TEST_BIND
 
 void TestWaylandRegistry::testRemoval()
@@ -380,6 +405,10 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(outputManagementAnnouncedSpy.isValid());
     QSignalSpy serverSideDecorationManagerAnnouncedSpy(&registry, &Registry::serverSideDecorationManagerAnnounced);
     QVERIFY(serverSideDecorationManagerAnnouncedSpy.isValid());
+    QSignalSpy blurAnnouncedSpy(&registry, &Registry::blurAnnounced);
+    QVERIFY(blurAnnouncedSpy.isValid());
+    QSignalSpy idleInhibitManagerUnstableV1AnnouncedSpy(&registry, &Registry::idleInhibitManagerUnstableV1Announced);
+    QVERIFY(idleInhibitManagerUnstableV1AnnouncedSpy.isValid());
 
     QVERIFY(!registry.isValid());
     registry.create(connection.display());
@@ -394,6 +423,9 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(!subCompositorAnnouncedSpy.isEmpty());
     QVERIFY(!outputManagementAnnouncedSpy.isEmpty());
     QVERIFY(!serverSideDecorationManagerAnnouncedSpy.isEmpty());
+    QVERIFY(!blurAnnouncedSpy.isEmpty());
+    QVERIFY(!idleInhibitManagerUnstableV1AnnouncedSpy.isEmpty());
+
 
     QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::Compositor));
     QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::Output));
@@ -405,6 +437,8 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(!registry.hasInterface(KWayland::Client::Registry::Interface::FullscreenShell));
     QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::OutputManagement));
     QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::ServerSideDecorationManager));
+    QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::Blur));
+    QVERIFY(registry.hasInterface(KWayland::Client::Registry::Interface::IdleInhibitManagerUnstableV1));
 
     QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::Compositor).isEmpty());
     QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::Output).isEmpty());
@@ -416,6 +450,8 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(registry.interfaces(KWayland::Client::Registry::Interface::FullscreenShell).isEmpty());
     QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::OutputManagement).isEmpty());
     QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::ServerSideDecorationManager).isEmpty());
+    QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::Blur).isEmpty());
+    QVERIFY(!registry.interfaces(KWayland::Client::Registry::Interface::IdleInhibitManagerUnstableV1).isEmpty());
 
     QSignalSpy seatRemovedSpy(&registry, SIGNAL(seatRemoved(quint32)));
     QVERIFY(seatRemovedSpy.isValid());
@@ -426,6 +462,9 @@ void TestWaylandRegistry::testRemoval()
     Compositor *compositor = registry.createCompositor(registry.interface(Registry::Interface::Compositor).name, registry.interface(Registry::Interface::Compositor).version, &registry);
     SubCompositor *subcompositor = registry.createSubCompositor(registry.interface(Registry::Interface::SubCompositor).name, registry.interface(Registry::Interface::SubCompositor).version, &registry);
     ServerSideDecorationManager *serverSideDeco = registry.createServerSideDecorationManager(registry.interface(Registry::Interface::ServerSideDecorationManager).name, registry.interface(Registry::Interface::ServerSideDecorationManager).version, &registry);
+    BlurManager *blurManager = registry.createBlurManager(registry.interface(Registry::Interface::Blur).name, registry.interface(Registry::Interface::Blur).version, &registry);
+    auto idleInhibitManager = registry.createIdleInhibitManager(registry.interface(Registry::Interface::IdleInhibitManagerUnstableV1).name, registry.interface(Registry::Interface::IdleInhibitManagerUnstableV1).version, &registry);
+
     connection.flush();
     m_display->dispatchEvents();
     QSignalSpy seatObjectRemovedSpy(seat, &Seat::removed);
@@ -438,6 +477,8 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(compositorObjectRemovedSpy.isValid());
     QSignalSpy subcompositorObjectRemovedSpy(subcompositor, &SubCompositor::removed);
     QVERIFY(subcompositorObjectRemovedSpy.isValid());
+    QSignalSpy idleInhibitManagerObjectRemovedSpy(idleInhibitManager, &IdleInhibitManager::removed);
+    QVERIFY(idleInhibitManagerObjectRemovedSpy.isValid());
 
     delete m_seat;
     QVERIFY(seatRemovedSpy.wait());
@@ -516,6 +557,26 @@ void TestWaylandRegistry::testRemoval()
     QVERIFY(registry.interfaces(KWayland::Client::Registry::Interface::ServerSideDecorationManager).isEmpty());
     QCOMPARE(serverSideDecoManagerObjectRemovedSpy.count(), 1);
 
+    QSignalSpy blurRemovedSpy(&registry, &Registry::blurRemoved);
+    QVERIFY(blurRemovedSpy.isValid());
+    QSignalSpy blurObjectRemovedSpy(blurManager, &BlurManager::removed);
+    QVERIFY(blurObjectRemovedSpy.isValid());
+
+    delete m_blur;
+    QVERIFY(blurRemovedSpy.wait());
+    QCOMPARE(blurRemovedSpy.first().first(), blurRemovedSpy.first().first());
+    QVERIFY(!registry.hasInterface(KWayland::Client::Registry::Interface::Blur));
+    QVERIFY(registry.interfaces(KWayland::Client::Registry::Interface::Blur).isEmpty());
+    QCOMPARE(blurObjectRemovedSpy.count(), 1);
+
+    QSignalSpy idleInhibitManagerUnstableV1RemovedSpy(&registry, &Registry::idleInhibitManagerUnstableV1Removed);
+    QVERIFY(idleInhibitManagerUnstableV1RemovedSpy.isValid());
+    delete m_idleInhibit;
+    QVERIFY(idleInhibitManagerUnstableV1RemovedSpy.wait());
+    QCOMPARE(idleInhibitManagerUnstableV1RemovedSpy.first().first(), idleInhibitManagerUnstableV1AnnouncedSpy.first().first());
+    QVERIFY(registry.interfaces(KWayland::Client::Registry::Interface::IdleInhibitManagerUnstableV1).isEmpty());
+    QCOMPARE(idleInhibitManagerObjectRemovedSpy.count(), 1);
+
     // cannot test shmRemoved as there is no functionality for it
 
     // verify everything has been removed only once
@@ -525,6 +586,82 @@ void TestWaylandRegistry::testRemoval()
     QCOMPARE(compositorObjectRemovedSpy.count(), 1);
     QCOMPARE(subcompositorObjectRemovedSpy.count(), 1);
     QCOMPARE(serverSideDecoManagerObjectRemovedSpy.count(), 1);
+    QCOMPARE(blurObjectRemovedSpy.count(), 1);
+    QCOMPARE(idleInhibitManagerObjectRemovedSpy.count(), 1);
+}
+
+void TestWaylandRegistry::testOutOfSyncRemoval()
+{
+    //This tests the following sequence of events
+    //server announces global
+    //client binds to that global
+
+    //server removes the global
+    //(simultaneously) the client legimitely uses the bound resource to the global
+    //client then gets the server events...it should no-op, not be a protcol error
+
+    using namespace KWayland::Client;
+    KWayland::Client::ConnectionThread connection;
+    QSignalSpy connectedSpy(&connection, SIGNAL(connected()));
+    connection.setSocketName(s_socketName);
+    connection.initConnection();
+    QVERIFY(connectedSpy.wait());
+    connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::aboutToBlock, &connection,
+        [&connection] {
+            wl_display_flush(connection.display());
+        }
+    );
+
+    KWayland::Client::Registry registry;
+    QVERIFY(!registry.isValid());
+    registry.create(connection.display());
+    registry.setup();
+    QSignalSpy blurAnnouncedSpy(&registry, &Registry::blurAnnounced);
+    QSignalSpy contrastAnnouncedSpy(&registry, &Registry::blurAnnounced);
+
+    blurAnnouncedSpy.wait();
+    contrastAnnouncedSpy.wait();
+    BlurManager *blurManager = registry.createBlurManager(registry.interface(Registry::Interface::Blur).name, registry.interface(Registry::Interface::Blur).version, &registry);
+    ContrastManager *contrastManager = registry.createContrastManager(registry.interface(Registry::Interface::Contrast).name, registry.interface(Registry::Interface::Contrast).version, &registry);
+
+    connection.flush();
+    m_display->dispatchEvents();
+
+    QScopedPointer<Compositor> compositor(registry.createCompositor(registry.interface(Registry::Interface::Compositor).name,
+                                             registry.interface(Registry::Interface::Compositor).version));
+    QScopedPointer<Surface> surface(compositor->createSurface());
+    QVERIFY(surface);
+
+    //remove blur
+    QSignalSpy blurRemovedSpy(&registry, &Registry::blurRemoved);
+
+    delete m_blur;
+
+    //client hasn't processed the event yet
+    QVERIFY(blurRemovedSpy.count() == 0);
+
+    //use the in the client
+    blurManager->createBlur(surface.data(), 0);
+
+    //now process events,
+    QVERIFY(blurRemovedSpy.wait());
+    QVERIFY(blurRemovedSpy.count() == 1);
+
+    //remove background contrast
+    QSignalSpy contrastRemovedSpy(&registry, &Registry::contrastRemoved);
+
+    delete m_contrast;
+
+    //client hasn't processed the event yet
+    QVERIFY(contrastRemovedSpy.count() == 0);
+
+    //use the in the client
+    contrastManager->createContrast(surface.data(), 0);
+
+    //now process events,
+    QVERIFY(contrastRemovedSpy.wait());
+    QVERIFY(contrastRemovedSpy.count() == 1);
+
 }
 
 void TestWaylandRegistry::testDestroy()
