@@ -133,10 +133,17 @@ void TestWaylandServerDisplay::testClientConnection()
     ClientConnection *connection = display.getConnection(client);
     QVERIFY(connection);
     QCOMPARE(connection->client(), client);
+    if (getuid() == 0) {
+        QEXPECT_FAIL("", "Please don't run test as root", Continue);
+    }
     QVERIFY(connection->userId() != 0);
+    if (getgid() == 0) {
+        QEXPECT_FAIL("", "Please don't run test as root", Continue);
+    }
     QVERIFY(connection->groupId() != 0);
     QVERIFY(connection->processId() != 0);
     QCOMPARE(connection->display(), &display);
+    QCOMPARE(connection->executablePath(), QCoreApplication::applicationFilePath());
     QCOMPARE((wl_client*)*connection, client);
     const ClientConnection &constRef = *connection;
     QCOMPARE((wl_client*)constRef, client);
@@ -169,7 +176,10 @@ void TestWaylandServerDisplay::testClientConnection()
     QVERIFY(disconnectedSpy.isEmpty());
     wl_client_destroy(client);
     QCOMPARE(disconnectedSpy.count(), 1);
+    QSignalSpy clientDestroyedSpy(client2, &QObject::destroyed);
+    QVERIFY(clientDestroyedSpy.isValid());
     client2->destroy();
+    QVERIFY(clientDestroyedSpy.wait());
     QCOMPARE(disconnectedSpy.count(), 2);
     close(sv[0]);
     close(sv[1]);
