@@ -26,14 +26,19 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // Wayland
 #include <wayland-plasma-window-management-client-protocol.h>
 
+#include <QtConcurrentRun>
+#include <QFutureWatcher>
 #include <QTimer>
+#include <qplatformdefs.h>
+
+#include <errno.h>
 
 namespace KWayland
 {
 namespace Client
 {
 
-class PlasmaWindowManagement::Private
+class Q_DECL_HIDDEN PlasmaWindowManagement::Private
 {
 public:
     Private(PlasmaWindowManagement *q);
@@ -55,7 +60,7 @@ private:
     PlasmaWindowManagement *q;
 };
 
-class PlasmaWindow::Private
+class Q_DECL_HIDDEN PlasmaWindow::Private
 {
 public:
     Private(org_kde_plasma_window *window, quint32 internalId, PlasmaWindow *q);
@@ -88,10 +93,12 @@ public:
     QPointer<PlasmaWindow> parentWindow;
     QMetaObject::Connection parentWindowUnmappedConnection;
     QRect geometry;
+    quint32 pid = 0;
 
 private:
     static void titleChangedCallback(void *data, org_kde_plasma_window *window, const char *title);
     static void appIdChangedCallback(void *data, org_kde_plasma_window *window, const char *app_id);
+    static void pidChangedCallback(void *data, org_kde_plasma_window *window, uint32_t pid);
     static void stateChangedCallback(void *data, org_kde_plasma_window *window, uint32_t state);
     static void virtualDesktopChangedCallback(void *data, org_kde_plasma_window *window, int32_t number);
     static void themedIconNameChangedCallback(void *data, org_kde_plasma_window *window, const char *name);
@@ -99,6 +106,7 @@ private:
     static void initialStateCallback(void *data, org_kde_plasma_window *window);
     static void parentWindowCallback(void *data, org_kde_plasma_window *window, org_kde_plasma_window *parent);
     static void windowGeometryCallback(void *data, org_kde_plasma_window *window, int32_t x, int32_t y, uint32_t width, uint32_t height);
+    static void iconChangedCallback(void *data, org_kde_plasma_window *org_kde_plasma_window);
     void setActive(bool set);
     void setMinimized(bool set);
     void setMaximized(bool set);
@@ -118,6 +126,7 @@ private:
     void setResizable(bool set);
     void setVirtualDesktopChangeable(bool set);
     void setParentWindow(PlasmaWindow *parentWindow);
+    void setPid(const quint32 pid);
 
     static Private *cast(void *data) {
         return reinterpret_cast<Private*>(data);
@@ -334,7 +343,9 @@ org_kde_plasma_window_listener PlasmaWindow::Private::s_listener = {
     unmappedCallback,
     initialStateCallback,
     parentWindowCallback,
-    windowGeometryCallback
+    windowGeometryCallback,
+    iconChangedCallback,
+    pidChangedCallback
 };
 
 void PlasmaWindow::Private::parentWindowCallback(void *data, org_kde_plasma_window *window, org_kde_plasma_window *parent)
@@ -415,6 +426,16 @@ void PlasmaWindow::Private::appIdChangedCallback(void *data, org_kde_plasma_wind
     emit p->q->appIdChanged();
 }
 
+void PlasmaWindow::Private::pidChangedCallback(void *data, org_kde_plasma_window *window, uint32_t pid)
+{
+    Q_UNUSED(window)
+    Private *p = cast(data);
+    if (p->pid == static_cast<quint32>(pid)) {
+        return;
+    }
+    p->pid = pid;
+}
+
 void PlasmaWindow::Private::virtualDesktopChangedCallback(void *data, org_kde_plasma_window *window, int32_t number)
 {
     Q_UNUSED(window)
@@ -473,6 +494,66 @@ void PlasmaWindow::Private::themedIconNameChangedCallback(void *data, org_kde_pl
     emit p->q->iconChanged();
 }
 
+static int readData(int fd, QByteArray &data)
+{
+    // implementation based on QtWayland file qwaylanddataoffer.cpp
+    char buf[4096];
+    int retryCount = 0;
+    int n;
+    while (true) {
+        n = QT_READ(fd, buf, sizeof buf);
+        if (n == -1 && (errno == EAGAIN) && ++retryCount < 1000) {
+            usleep(1000);
+        } else {
+            break;
+        }
+    }
+    if (n > 0) {
+        data.append(buf, n);
+        n = readData(fd, data);
+    }
+    return n;
+}
+
+void PlasmaWindow::Private::iconChangedCallback(void *data, org_kde_plasma_window *window)
+{
+    auto p = cast(data);
+    Q_UNUSED(window);
+    int pipeFds[2];
+    if (pipe2(pipeFds, O_CLOEXEC|O_NONBLOCK) != 0) {
+        return;
+    }
+    org_kde_plasma_window_get_icon(p->window, pipeFds[1]);
+    close(pipeFds[1]);
+    const int pipeFd = pipeFds[0];
+    auto readIcon = [pipeFd] () -> QIcon {
+        QByteArray content;
+        if (readData(pipeFd, content) != 0) {
+            close(pipeFd);
+            return QIcon();
+        }
+        close(pipeFd);
+        QDataStream ds(content);
+        QIcon icon;
+        ds >> icon;
+        return icon;
+    };
+    QFutureWatcher<QIcon> *watcher = new QFutureWatcher<QIcon>(p->q);
+    QObject::connect(watcher, &QFutureWatcher<QIcon>::finished, p->q,
+        [p, watcher] {
+            watcher->deleteLater();
+            QIcon icon = watcher->result();
+            if (!icon.isNull()) {
+                p->icon = icon;
+            } else {
+                p->icon = QIcon::fromTheme(QStringLiteral("wayland"));
+            }
+            emit p->q->iconChanged();
+        }
+    );
+    watcher->setFuture(QtConcurrent::run(readIcon));
+}
+
 void PlasmaWindow::Private::setActive(bool set)
 {
     if (active == set) {
@@ -684,6 +765,11 @@ QString PlasmaWindow::appId() const
     return d->appId;
 }
 
+quint32 PlasmaWindow::pid() const
+{
+    return d->pid;
+}
+
 QString PlasmaWindow::title() const
 {
     return d->title;
@@ -817,6 +903,32 @@ void PlasmaWindow::requestVirtualDesktop(quint32 desktop)
     org_kde_plasma_window_set_virtual_desktop(d->window, desktop);
 }
 
+void PlasmaWindow::requestToggleKeepAbove()
+{
+    if (d->keepAbove) {
+        org_kde_plasma_window_set_state(d->window,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE,
+            0);
+    } else {
+        org_kde_plasma_window_set_state(d->window,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE);
+    }
+}
+
+void PlasmaWindow::requestToggleKeepBelow()
+{
+    if (d->keepBelow) {
+        org_kde_plasma_window_set_state(d->window,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW,
+            0);
+    } else {
+        org_kde_plasma_window_set_state(d->window,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW,
+            ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW);
+    }
+}
+
 void PlasmaWindow::requestToggleMinimized()
 {
     if (d->minimized) {
