@@ -53,6 +53,8 @@ class CompositorInterface;
 class DataDeviceManagerInterface;
 class DpmsManagerInterface;
 class IdleInterface;
+enum class IdleInhibitManagerInterfaceVersion;
+class IdleInhibitManagerInterface;
 class FakeInputInterface;
 class OutputInterface;
 class OutputDeviceInterface;
@@ -81,6 +83,8 @@ class PointerGesturesInterface;
 enum class PointerConstraintsInterfaceVersion;
 class PointerConstraintsInterface;
 class XdgForeignInterface;
+class AppMenuManagerInterface;
+class ServerSideDecorationPaletteManagerInterface;
 
 /**
  * @brief Class holding the Wayland server display loop.
@@ -227,6 +231,32 @@ public:
      * @since 5.40
      **/
     XdgForeignInterface *createXdgForeignInterface(QObject *parent = nullptr);
+
+    /**
+     * Creates the IdleInhibitManagerInterface in interface @p version.
+     *
+     * @returns The created manager object
+     * @since 5.41
+     **/
+    IdleInhibitManagerInterface *createIdleInhibitManager(const IdleInhibitManagerInterfaceVersion &version, QObject *parent = nullptr);
+
+    /**
+     * Creates the AppMenuManagerInterface in interface @p version.
+     *
+     * @returns The created manager object
+     * @since 5.42
+     **/
+    AppMenuManagerInterface *createAppMenuManagerInterface(QObject *parent = nullptr);
+
+    /**
+     * Creates the ServerSideDecorationPaletteManagerInterface in interface @p version.
+     *
+     * @returns The created manager object
+     * @since 5.42
+     **/
+    ServerSideDecorationPaletteManagerInterface *createServerSideDecorationPaletteManager(QObject *parent = nullptr);
+
+
     /**
      * Gets the ClientConnection for the given @p client.
      * If there is no ClientConnection yet for the given @p client, it will be created.
