@@ -18,9 +18,14 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "pointer_interface.h"
+#include "pointer_interface_p.h"
+#include "pointerconstraints_interface.h"
+#include "pointergestures_interface_p.h"
 #include "resource_p.h"
+#include "relativepointer_interface_p.h"
 #include "seat_interface.h"
 #include "display.h"
+#include "subcompositor_interface.h"
 #include "surface_interface.h"
 // Wayland
 #include <wayland-server.h>
@@ -31,30 +36,6 @@ namespace KWayland
 namespace Server
 {
 
-class PointerInterface::Private : public Resource::Private
-{
-public:
-    Private(SeatInterface *parent, wl_resource *parentResource, PointerInterface *q);
-
-    SeatInterface *seat;
-    SurfaceInterface *focusedSurface = nullptr;
-    QMetaObject::Connection destroyConnection;
-    Cursor *cursor = nullptr;
-
-private:
-    PointerInterface *q_func() {
-        return reinterpret_cast<PointerInterface *>(q);
-    }
-    void setCursor(quint32 serial, SurfaceInterface *surface, const QPoint &hotspot);
-    // interface
-    static void setCursorCallback(wl_client *client, wl_resource *resource, uint32_t serial,
-                                  wl_resource *surface, int32_t hotspot_x, int32_t hotspot_y);
-    // since version 3
-    static void releaseCallback(wl_client *client, wl_resource *resource);
-
-    static const struct wl_pointer_interface s_interface;
-};
-
 class Cursor::Private
 {
 public:
@@ -89,15 +70,156 @@ void PointerInterface::Private::setCursor(quint32 serial, SurfaceInterface *surf
     }
 }
 
+void PointerInterface::Private::sendLeave(SurfaceInterface *surface, quint32 serial)
+{
+    if (!surface) {
+        return;
+    }
+    if (resource && surface->resource()) {
+        wl_pointer_send_leave(resource, serial, surface->resource());
+    }
+}
+
+void PointerInterface::Private::registerRelativePointer(RelativePointerInterface *relativePointer)
+{
+    relativePointers << relativePointer;
+    QObject::connect(relativePointer, &QObject::destroyed, q,
+        [this, relativePointer] {
+            relativePointers.removeOne(relativePointer);
+        }
+    );
+}
+
+
+void PointerInterface::Private::registerSwipeGesture(PointerSwipeGestureInterface *gesture)
+{
+    swipeGestures << gesture;
+    QObject::connect(gesture, &QObject::destroyed, q,
+        [this, gesture] {
+            swipeGestures.removeOne(gesture);
+        }
+    );
+}
+
+void PointerInterface::Private::registerPinchGesture(PointerPinchGestureInterface *gesture)
+{
+    pinchGestures << gesture;
+    QObject::connect(gesture, &QObject::destroyed, q,
+        [this, gesture] {
+            pinchGestures.removeOne(gesture);
+        }
+    );
+}
+
+namespace {
+static QPointF surfacePosition(SurfaceInterface *surface) {
+    if (surface && surface->subSurface()) {
+        return surface->subSurface()->position() + surfacePosition(surface->subSurface()->parentSurface().data());
+    }
+    return QPointF();
+}
+}
+
+void PointerInterface::Private::sendEnter(SurfaceInterface *surface, const QPointF &parentSurfacePosition, quint32 serial)
+{
+    if (!surface || !surface->resource()) {
+        return;
+    }
+    const QPointF adjustedPos = parentSurfacePosition - surfacePosition(surface);
+    wl_pointer_send_enter(resource, serial,
+                          surface->resource(),
+                          wl_fixed_from_double(adjustedPos.x()), wl_fixed_from_double(adjustedPos.y()));
+}
+
+void PointerInterface::Private::startSwipeGesture(quint32 serial, quint32 fingerCount)
+{
+    if (swipeGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = swipeGestures.constBegin(), end = swipeGestures.constEnd(); it != end; it++) {
+        (*it)->start(serial, fingerCount);
+    }
+}
+
+void PointerInterface::Private::updateSwipeGesture(const QSizeF &delta)
+{
+    if (swipeGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = swipeGestures.constBegin(), end = swipeGestures.constEnd(); it != end; it++) {
+        (*it)->update(delta);
+    }
+}
+
+void PointerInterface::Private::endSwipeGesture(quint32 serial)
+{
+    if (swipeGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = swipeGestures.constBegin(), end = swipeGestures.constEnd(); it != end; it++) {
+        (*it)->end(serial);
+    }
+}
+
+void PointerInterface::Private::cancelSwipeGesture(quint32 serial)
+{
+    if (swipeGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = swipeGestures.constBegin(), end = swipeGestures.constEnd(); it != end; it++) {
+        (*it)->cancel(serial);
+    }
+}
+
+void PointerInterface::Private::startPinchGesture(quint32 serial, quint32 fingerCount)
+{
+    if (pinchGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = pinchGestures.constBegin(), end = pinchGestures.constEnd(); it != end; it++) {
+        (*it)->start(serial, fingerCount);
+    }
+}
+
+void PointerInterface::Private::updatePinchGesture(const QSizeF &delta, qreal scale, qreal rotation)
+{
+    if (pinchGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = pinchGestures.constBegin(), end = pinchGestures.constEnd(); it != end; it++) {
+        (*it)->update(delta, scale, rotation);
+    }
+}
+
+void PointerInterface::Private::endPinchGesture(quint32 serial)
+{
+    if (pinchGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = pinchGestures.constBegin(), end = pinchGestures.constEnd(); it != end; it++) {
+        (*it)->end(serial);
+    }
+}
+
+void PointerInterface::Private::cancelPinchGesture(quint32 serial)
+{
+    if (pinchGestures.isEmpty()) {
+        return;
+    }
+    for (auto it = pinchGestures.constBegin(), end = pinchGestures.constEnd(); it != end; it++) {
+        (*it)->cancel(serial);
+    }
+}
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_pointer_interface PointerInterface::Private::s_interface = {
     setCursorCallback,
-    releaseCallback
+    resourceDestroyedCallback
 };
 #endif
 
 PointerInterface::PointerInterface(SeatInterface *parent, wl_resource *parentResource)
-    : Resource(new Private(parent, parentResource, this), parent)
+    : Resource(new Private(parent, parentResource, this))
 {
     // TODO: handle touch
     connect(parent, &SeatInterface::pointerPosChanged, this, [this] {
@@ -107,9 +229,25 @@ PointerInterface::PointerInterface(SeatInterface *parent, wl_resource *parentRes
             return;
         }
         if (d->focusedSurface && d->resource) {
+            if (!d->focusedSurface->lockedPointer().isNull() && d->focusedSurface->lockedPointer()->isLocked()) {
+                return;
+            }
             const QPointF pos = d->seat->focusedPointerSurfaceTransformation().map(d->seat->pointerPos());
-            wl_pointer_send_motion(d->resource, d->seat->timestamp(),
-                                   wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+            auto targetSurface = d->focusedSurface->surfaceAt(pos);
+            if (!targetSurface) {
+                targetSurface = d->focusedSurface;
+            }
+            if (targetSurface != d->focusedChildSurface.data()) {
+                const quint32 serial = d->seat->display()->nextSerial();
+                d->sendLeave(d->focusedChildSurface.data(), serial);
+                d->focusedChildSurface = QPointer<SurfaceInterface>(targetSurface);
+                d->sendEnter(targetSurface, pos, serial);
+                d->client->flush();
+            } else {
+                const QPointF adjustedPos = pos - surfacePosition(d->focusedChildSurface);
+                wl_pointer_send_motion(d->resource, d->seat->timestamp(),
+                                       wl_fixed_from_double(adjustedPos.x()), wl_fixed_from_double(adjustedPos.y()));
+            }
         }
     });
 }
@@ -119,28 +257,29 @@ PointerInterface::~PointerInterface() = default;
 void PointerInterface::setFocusedSurface(SurfaceInterface *surface, quint32 serial)
 {
     Q_D();
-    if (d->focusedSurface) {
-        if (d->resource && d->focusedSurface->resource()) {
-            wl_pointer_send_leave(d->resource, serial, d->focusedSurface->resource());
-        }
-        disconnect(d->destroyConnection);
-    }
+    d->sendLeave(d->focusedChildSurface.data(), serial);
+    disconnect(d->destroyConnection);
     if (!surface) {
         d->focusedSurface = nullptr;
+        d->focusedChildSurface.clear();
         return;
     }
     d->focusedSurface = surface;
-    d->destroyConnection = connect(d->focusedSurface, &QObject::destroyed, this,
+    d->destroyConnection = connect(d->focusedSurface, &Resource::aboutToBeUnbound, this,
         [this] {
             Q_D();
+            d->sendLeave(d->focusedChildSurface.data(), d->global->display()->nextSerial());
             d->focusedSurface = nullptr;
+            d->focusedChildSurface.clear();
         }
     );
 
     const QPointF pos = d->seat->focusedPointerSurfaceTransformation().map(d->seat->pointerPos());
-    wl_pointer_send_enter(d->resource, serial,
-                          d->focusedSurface->resource(),
-                          wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+    d->focusedChildSurface = QPointer<SurfaceInterface>(d->focusedSurface->surfaceAt(pos));
+    if (!d->focusedChildSurface) {
+        d->focusedChildSurface = QPointer<SurfaceInterface>(d->focusedSurface);
+    }
+    d->sendEnter(d->focusedChildSurface.data(), pos, serial);
     d->client->flush();
 }
 
@@ -184,23 +323,34 @@ void PointerInterface::Private::setCursorCallback(wl_client *client, wl_resource
     p->setCursor(serial, SurfaceInterface::get(surface), QPoint(hotspot_x, hotspot_y));
 }
 
-void PointerInterface::Private::releaseCallback(wl_client *client, wl_resource *resource)
-{
-    Q_UNUSED(client)
-    unbind(resource);
-}
-
 Cursor *PointerInterface::cursor() const
 {
     Q_D();
     return d->cursor;
 }
 
+void PointerInterface::relativeMotion(const QSizeF &delta, const QSizeF &deltaNonAccelerated, quint64 microseconds)
+{
+    Q_D();
+    if (d->relativePointers.isEmpty()) {
+        return;
+    }
+    for (auto it = d->relativePointers.constBegin(), end = d->relativePointers.constEnd(); it != end; it++) {
+        (*it)->relativeMotion(delta, deltaNonAccelerated, microseconds);
+    }
+    client()->flush();
+}
+
 PointerInterface::Private *PointerInterface::d_func() const
 {
     return reinterpret_cast<Private*>(d.data());
 }
 
+PointerInterface *PointerInterface::get(wl_resource *native)
+{
+    return Private::get<PointerInterface>(native);
+}
+
 Cursor::Private::Private(Cursor *q, PointerInterface *pointer)
     : pointer(pointer)
     , q(q)
